/************************************************************************************
 * WrightEagle (Soccer Simulation League 2D)                                        *
 * BASE SOURCE CODE RELEASE 2010                                                    *
 * Copyright (C) 1998-2010 WrightEagle 2D Soccer Simulation Team,                   *
 *                         Multi-Agent Systems Lab.,                                *
 *                         School of Computer Science and Technology,               *
 *                         University of Science and Technology of China, China.    *
 *                                                                                  *
 * This program is free software; you can redistribute it and/or                    *
 * modify it under the terms of the GNU General Public License                      *
 * as published by the Free Software Foundation; either version 2                   *
 * of the License, or (at your option) any later version.                           *
 *                                                                                  *
 * This program is distributed in the hope that it will be useful,                  *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                   *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                    *
 * GNU General Public License for more details.                                     *
 *                                                                                  *
 * You should have received a copy of the GNU General Public License                *
 * along with this program; if not, write to the Free Software                      *
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.  *
 ************************************************************************************/

#include "Client.h"
#include "DynamicDebug.h"
#include "Formation.h"
#include "Kicker.h"
#include "CommandSender.h"
#include "Parser.h"
#include "Thread.h"
#include "UDPSocket.h"
#include "WorldModel.h"
#include "Agent.h"
#include "Logger.h"
#include "CommunicateSystem.h"
#include "NetworkTest.h"
#include "Dasher.h"
#include "Tackler.h"
#include "TimeTest.h"
#include "VisualSystem.h"
#include "InterceptModel.h"
#include "Plotter.h"

Client::Client() {
	srand(time(0)); //global srand once
	srand48(time(0));

	/** Observer and World Model */
	mpAgent         = 0;
    mpObserver      = new Observer;
	mpWorldModel    = new WorldModel;

	/** Parser thread and CommandSend thread */
	mpParser        = new Parser(mpObserver);
	mpCommandSender = new CommandSender(mpObserver);

    // instanceﴴԽʡЧʣinstanceܷҪʱ

    /** Assistant instance */
	TimeTest::instance();
    NetworkTest::instance();
    DynamicDebug::instance();
    Logger::instance().Initial(mpObserver, &(mpWorldModel->World(false)));
    Plotter::instance();

    /** Param */
    ServerParam::instance();
    PlayerParam::instance();

	/** Base Model */
    InterceptModel::instance();

    /** Smart Action */
    Dasher::instance();
    Tackler::instance();
    Kicker::instance();

    /** Other Useful instance */
    BehaviorFactory::instance();
    UDPSocket::instance();
    VisualSystem::instance();
    CommunicateSystem::instance();
}

Client::~Client()
{
	delete mpCommandSender;
	delete mpParser;

	delete mpObserver;
	delete mpWorldModel;
	delete mpAgent;
}

void Client::RunDynamicDebug()
{
	static char msg[MAX_MESSAGE];
	DynamicDebug::instance().Initial(mpObserver); // ̬Եĳʼעλòƶ

	DynamicDebug::instance().Run(msg); // ʼϢ
	mpParser->ParseInitializeMsg(msg);

	ConstructAgent();

	MessageType msg_type;

	bool first_parse = true;

	while (true)
	{
		msg_type = DynamicDebug::instance().Run(msg);

		switch (msg_type)
		{
		case MT_Parse:
			if (first_parse) {
				mpObserver->Reset();
				first_parse = false;
			}
			mpParser->Parse(msg);
			break;
		case MT_Run:
			Run();
			Logger::instance().Flush(); //flush log
			mpObserver->SetPlanned();
			break;
		case MT_Send:
			mpCommandSender->Run();
			first_parse = true;
			break;
		default:
			return;
		}
	}
}

void Client::RunNormal()
{
	mpParser->Start(); //̣߳serverϢ
	mpCommandSender->Start(); //̣߳serverϢ
	Logger::instance().Start(); //log߳

	int past_cycle = 0;
	do
	{
		WaitFor(100); // wait for the parser thread to connect the server
		if (++past_cycle > 20)
		{
			std::cout << PlayerParam::instance().teamName() << ": Connect Server Error ..." << std::endl;
			return;
		}
	} while (mpParser->IsConnectServerOk() == false);

	ConstructAgent();

	SendOptionToServer();

	MainLoop();

	WaitFor(mpObserver->MyUnum() * 100);
    if (mpObserver->MyUnum() == 0)
    {
        std::cout << PlayerParam::instance().teamName() << " Coach: Bye ..." << std::endl;
    }
    else
    {
        std::cout << PlayerParam::instance().teamName() << " " << mpObserver->MyUnum() << ": Bye ..." << std::endl;
    }
}

void Client::ConstructAgent()
{
	Assert(mpAgent == 0);
	mpAgent = new Agent(mpObserver->MyUnum(), mpWorldModel, false); //Ҫ֪ܳʼ

	Formation::instance.AssignWith(mpAgent);
	mpCommandSender->RegisterAgent(mpAgent);
	CommunicateSystem::instance().Initial(mpObserver , mpAgent); //init communicate system
	VisualSystem::instance().Initial(mpAgent);
}

void Client::MainLoop()
{
	while (mpObserver->WaitForNewInfo()) // ȴӾ
	{
        NetworkTest::instance().AddDecisionBegin();

        // bye ...
        if (mpObserver->GetPlayMode() == PM_Time_Over)
        {
            mpAgent->CheckCommands(mpObserver);
            mpAgent->Bye();

            mpObserver->SetPlanned();
            mpObserver->SetCommandSend();
            Logger::instance().SetFlushCond();

            break;
        }

        DynamicDebug::instance().AddMessage("\0", MT_Run); // ̬Լ¼RunϢ
        Run();

		mpObserver->SetPlanned();
		mpObserver->SetCommandSend(); //ѷ߳
		Logger::instance().SetFlushCond(); // set flush cond and let the logger thread flush the logs to file.

        NetworkTest::instance().AddDecisionEnd(mpObserver->CurrentTime());
	}
}
