/************************************************************************************
 * WrightEagle (Soccer Simulation League 2D)                                        *
 * BASE SOURCE CODE RELEASE 2010                                                    *
 * Copyright (C) 1998-2010 WrightEagle 2D Soccer Simulation Team,                   *
 *                         Multi-Agent Systems Lab.,                                *
 *                         School of Computer Science and Technology,               *
 *                         University of Science and Technology of China, China.    *
 *                                                                                  *
 * This program is free software; you can redistribute it and/or                    *
 * modify it under the terms of the GNU General Public License                      *
 * as published by the Free Software Foundation; either version 2                   *
 * of the License, or (at your option) any later version.                           *
 *                                                                                  *
 * This program is distributed in the hope that it will be useful,                  *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                   *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                    *
 * GNU General Public License for more details.                                     *
 *                                                                                  *
 * You should have received a copy of the GNU General Public License                *
 * along with this program; if not, write to the Free Software                      *
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.  *
 ************************************************************************************/

#ifndef VISUALSYSTEM_H_
#define VISUALSYSTEM_H_

#include <cstring>
#include <list>
#include <fstream>

#include "Types.h"
#include "Utilities.h"
#include "Geometry.h"
#include "PlayerState.h"
#include "ServerParam.h"
#include <set>

class MobileState;
class PlayerState;
class BallState;
class Observer;
class Agent;
class WorldState;
class InfoState;

/**
* ΪӾϵͳӾӾϵͳӾ
*/
class VisualSystem {
	VisualSystem();

	class VisualRequest
	{
	public:
		VisualRequest(): mPrePos(Vector(HUGE_VALUE, HUGE_VALUE)), mCycleDelay(0), mConf(0.0), mpObject(0)
		{
			Clear();
		}

		void Clear() {
			mValid = false;
			mFreq = 100.0;
			mScore = 0.0;
		}

		Vector mPrePos;

		double PreDistance() {
			return mPrePos.Mod();
		}

		bool mValid;
		int  mCycleDelay;

		double mFreq;    //ӾҪӾƵ
		double mConf;    //ƵڵŶ
		double mScore;   //Ϣ

		const MobileState *mpObject;

		int mUnum; //+, -, 0

		bool operator > (const VisualRequest &o){
			return mConf == o.mConf?
					mCycleDelay == o.mCycleDelay?
							mFreq < o.mFreq
							: mCycleDelay > o.mCycleDelay
							  : mConf < o.mConf;
		}

		class PointerGreater {
		public:
			bool operator()(VisualRequest *p, VisualRequest *q){ //壺p < q if *p > *q;
				return *p > *q;
			}
		};

		void UpdateEvaluation() {
		    mFreq = Max(1.0, mFreq);
			mConf = Max(1.0 - (mCycleDelay / mFreq), 0.0);
			mScore = 1.0 - mConf;
			mScore = Max(mScore, 0.001);
		}

		double Multi() {
			return 1.0 + mScore;
		}
	};

	struct VisualAction
	{
		AngleDeg mDir;
		double   mScore;

		VisualAction(AngleDeg dir = 0.0, double score = -1.0): mDir(dir), mScore(score) {}
	};

	/**
	* ӾΧһܵ۷ֲ -- ȷһ
	*/
	class VisualRing {
	public:
		VisualRing() {
			Clear();
		}

		double & Score(const AngleDeg & ang) {
			return mScore[static_cast<int>(GetNormalizeAngleDeg(ang, 0.0))];
		}

		VisualAction GetBestVisualAction(const AngleDeg left_most, const AngleDeg right_most, const AngleDeg interval_length);

		void Dump(std::ostream & os, AngleDeg left_most, AngleDeg right_most) {
			double sum = 0.0;
			for (AngleDeg a = left_most; a < right_most; ++a) {
				double score = Score(a);
				os << a << ' ' << score << std::endl;
				sum += score;
			}
			std::cerr << "sum score: " << sum << std::endl;
		}

		void Show(AngleDeg left_most, AngleDeg right_most) {
			std::ofstream file("pp");
			Dump(file, left_most, right_most);
			file.close();

			Plotter::instance().GnuplotExecute("set xrange [%g:%g]", left_most, right_most);
			Plotter::instance().GnuplotExecute("plot \"pp\" with steps");
		}

		void Clear() {
			mScore.bzero();
		}

	private:
		Array<double, 361> mScore;
	};

public:
	static VisualSystem & instance();
	virtual ~VisualSystem();

	void ResetVisualRequest();
	void Decision();

	void RaiseBall(Agent & agent, double eva = 0.0) { if (mpAgent == & agent) { RaiseBall(eva); } }
	void RaisePlayer(Agent & agent, Unum num, double eva = 0.0) { if (mpAgent == & agent) { RaisePlayer(num, eva); } }

	void SetForceSeeBall(const Agent & agent) { if (mpAgent == & agent) { SetForceSeeBall(); } }
	void SetForceSeePlayer(const Agent & agent, Unum i) { if (mpAgent == & agent) { SetForceSeePlayer(i); } }

	void SetCritical(const Agent & agent, bool critical) { if (mpAgent == & agent) { SetCritical(critical); } }
	void ForbidDecision(const Agent & agent) { if (mpAgent == & agent) { mForbidden = true;} }
	void ChangeViewWidth(const Agent & agent, ViewWidth view_width) { if (mpAgent == & agent) { ChangeViewWidth(view_width); } }
	void SetCanTurn(const Agent & agent, bool can_turn) { if (mpAgent == & agent) { SetCanTurn(can_turn); } }

public:
	void Initial(Agent * agent);

private:
	void UpdatePredictInfo();
	void RaiseBall(double eva = 0.0);
	void RaisePlayer(ObjectIndex num, double eva = 0.0);
	void RaiseForgotObject(ObjectIndex unum);

	void ViewModeDecision();
	void DoInfoGather();
	void DealVisualRequest();
	void DealWithSpecialObjects();
	void SetVisualRing();
	void GetBestVisualAction();
	VisualAction GetBestVisualActionWithViewWidth(ViewWidth view_width, bool force = false);
	bool ForceSearchBall();
	void DoVisualExecute();
	void DoDecision();

	void SetForceSeeBall();
	void SetForceSeePlayer(Unum i);

	void SetCritical(bool critical) { mIsCritical = critical; }
	void ChangeViewWidth(ViewWidth view_width) { mViewWidth = view_width; }
	void SetCanTurn(bool can_turn) { mCanTurn = can_turn; }

	/**
	* Ӿ󣬼score
	*/
	void EvaluateVisualRequest();

	void DoInfoGatherForBallFree();
	void DoInfoGatherForFastForward();
	void DoInfoGatherForPenaltyAttack();
	void DoInfoGatherForDefense();
	void DoInfoGatherForGoalie();

	bool DealWithSetPlayMode();

	int NewSightComeCycle(ViewWidth view_width) { //Ӿ
		return Max(sight::SightDelay(view_width) - mpSelfState->GetPosDelay(), 1);
	}

	int NewSightWaitCycle(ViewWidth view_width) { //Ӿʱȴ
		return NewSightComeCycle(view_width) + mpSelfState->GetPosDelay();
	}

	VisualRequest * GetVisualRequest(Unum i) {
		Assert(i != 0);
		return & mVisualRequest[i];
	}

private:
	Agent       * mpAgent;
	WorldState  * mpWorldState;
	BallState   * mpBallState;
	PlayerState * mpSelfState;
	InfoState   * mpInfoState;

	ObjectArray<VisualRequest> mVisualRequest;
	ObjectArray<bool> mForceToSeeObject;
	std::set<VisualRequest *> mHighPriorityPlayerSet;

	bool mIsCritical;
	bool mCanForceChangeViewWidth;
	bool mIsSearching;
	bool mForbidden;

	bool mCanTurn;
	ViewWidth mViewWidth;			//ǰӾ
	AngleDeg  mPreBodyDir;		//ǰڶִǶȣturn
	Vector    mPreBallPos;				//ǰڶִλãkick
	Vector    mPreSelfPos;          //ǰڶִλãdash

	VisualAction mBestVisualAction;
	VisualRing mVisualRing;
	int mSenseBallCycle; //ں3mΧ

private:
	int GetSenseBallCycle();

private:
	class VirtualSelf: public PlayerState {
	public:
		VirtualSelf(const PlayerState & player): PlayerState(player) {}
		const double & GetKickableArea() const { return ServerParam::instance().visibleDistance(); }
	};
};

#endif /* VISUALSYSTEM_H_ */
